@@ -12,11 +12,13 @@ from itertools import chain, repeat
 from threading import RLock
 
 try:
+    from psycopg2 import connect
+except ImportError:
     from psycopg2cffi import compat
     compat.register()
 except ImportError:
-    pass
-from psycopg2 import connect, Binary
+    from psycopg2 import connect
+from psycopg2 import Binary
 from psycopg2.sql import SQL, Identifier
 from psycopg2.pool import ThreadedConnectionPool, PoolError
 from psycopg2.extensions import cursor
@@ -603,17 +605,16 @@ class Database(DatabaseInterface):
             version = self.get_version(connection)
         finally:
             self.put_connection(connection)
-        if version >= (11, 0):
-            ToTsQuery = WebsearchToTsQuery
-        else:
-            ToTsQuery = PlainToTsQuery
-        if language:
-            config_name = self._search_full_text_language(language)
-            if not isinstance(query, TsQuery):
-                query = ToTsQuery(config_name, query)
-        else:
-            if not isinstance(query, TsQuery):
-                query = ToTsQuery(query)
+        if not isinstance(query, TsQuery):
+            if version >= (11, 0):
+                ToTsQuery = WebsearchToTsQuery
+            else:
+                ToTsQuery = PlainToTsQuery
+            if language:
+                config_name = self._search_full_text_language(language)
+            else:
+                config_name = 'simple'
+            query = ToTsQuery(config_name, query)
         return query
 
     def search_full_text(self, document, query):
