@@ -3,13 +3,15 @@
 import re
 import logging
 
+from psycopg2.sql import SQL, Identifier, Composed
+
 from trytond.transaction import Transaction
 from trytond.backend.table import TableHandlerInterface
 
 __all__ = ['TableHandler']
 
 logger = logging.getLogger(__name__)
-VARCHAR_SIZE_RE = re.compile('VARCHAR\(([0-9]+)\)')
+VARCHAR_SIZE_RE = re.compile(r'VARCHAR\(([0-9]+)\)')
 
 
 class TableHandler(TableHandlerInterface):
@@ -33,7 +35,8 @@ class TableHandler(TableHandlerInterface):
 
         # Create new table if necessary
         if not self.table_exist(self.table_name):
-            cursor.execute('CREATE TABLE "%s" ()' % self.table_name)
+            cursor.execute(SQL('CREATE TABLE {} ()').format(
+                    Identifier(self.table_name)))
         self.table_schema = transaction.database.get_table_schema(
             transaction.connection, self.table_name)
 
@@ -43,40 +46,52 @@ class TableHandler(TableHandlerInterface):
         self.is_owner, = cursor.fetchone()
 
         if model.__doc__ and self.is_owner:
-            cursor.execute('COMMENT ON TABLE "%s" IS \'%s\'' %
-                (self.table_name, model.__doc__.replace("'", "''")))
+            cursor.execute(SQL('COMMENT ON TABLE {} IS %s').format(
+                        Identifier(self.table_name)),
+                (model.__doc__,))
 
         self._update_definitions(columns=True)
         if 'id' not in self._columns:
             if not self.history:
-                cursor.execute('ALTER TABLE "%s" '
-                    'ADD COLUMN id INTEGER '
-                    'DEFAULT nextval(\'"%s"\') NOT NULL'
-                    % (self.table_name, self.sequence_name))
-                cursor.execute('ALTER TABLE "%s" '
-                    'ADD PRIMARY KEY(id)' % self.table_name)
+                cursor.execute(
+                    SQL(
+                        "ALTER TABLE {} ADD COLUMN id INTEGER "
+                        "DEFAULT nextval(%s) NOT NULL").format(
+                        Identifier(self.table_name)),
+                    (self.sequence_name,))
+                cursor.execute(
+                    SQL('ALTER TABLE {} ADD PRIMARY KEY(id)')
+                    .format(Identifier(self.table_name)))
             else:
-                cursor.execute('ALTER TABLE "%s" '
-                    'ADD COLUMN id INTEGER' % self.table_name)
+                cursor.execute(
+                    SQL('ALTER TABLE {} ADD COLUMN id INTEGER')
+                    .format(Identifier(self.table_name)))
             self._update_definitions(columns=True)
         if self.history and '__id' not in self._columns:
-            cursor.execute('ALTER TABLE "%s" '
-                'ADD COLUMN __id INTEGER '
-                'DEFAULT nextval(\'"%s"\') NOT NULL' %
-                (self.table_name, self.sequence_name))
-            cursor.execute('ALTER TABLE "%s" '
-                'ADD PRIMARY KEY(__id)' % self.table_name)
+            cursor.execute(
+                SQL(
+                    "ALTER TABLE {} ADD COLUMN __id INTEGER "
+                    "DEFAULT nextval(%s) NOT NULL").format(
+                        Identifier(self.table_name)),
+                (self.sequence_name,))
+            cursor.execute(
+                SQL('ALTER TABLE {} ADD PRIMARY KEY(__id)')
+                .format(Identifier(self.table_name)))
         else:
             default = "nextval('%s'::regclass)" % self.sequence_name
             if self.history:
                 if self._columns['__id']['default'] != default:
-                    cursor.execute('ALTER TABLE "%s" '
-                        'ALTER __id SET DEFAULT %s'
-                        % (self.table_name, default))
+                    cursor.execute(
+                        SQL("ALTER TABLE {} "
+                            "ALTER __id SET DEFAULT nextval(%s::regclass)")
+                        .format(Identifier(self.table_name)),
+                        (self.sequence_name,))
             if self._columns['id']['default'] != default:
-                    cursor.execute('ALTER TABLE "%s" '
-                        'ALTER id SET DEFAULT %s'
-                        % (self.table_name, default))
+                cursor.execute(
+                    SQL("ALTER TABLE {} "
+                        "ALTER id SET DEFAULT nextval(%s::regclass)")
+                    .format(Identifier(self.table_name)),
+                    (self.sequence_name,))
         self._update_definitions()
 
     @staticmethod
@@ -92,8 +107,8 @@ class TableHandler(TableHandlerInterface):
         # Rename table
         if (TableHandler.table_exist(old_name)
                 and not TableHandler.table_exist(new_name)):
-            cursor.execute('ALTER TABLE "%s" RENAME TO "%s"'
-                % (old_name, new_name))
+            cursor.execute(SQL('ALTER TABLE {} RENAME TO {}').format(
+                    Identifier(old_name), Identifier(new_name)))
         # Rename sequence
         old_sequence = old_name + '_id_seq'
         new_sequence = new_name + '_id_seq'
@@ -110,19 +125,20 @@ class TableHandler(TableHandlerInterface):
     def column_exist(self, column_name):
         return column_name in self._columns
 
-    def column_rename(self, old_name, new_name, exception=False):
+    def column_rename(self, old_name, new_name):
         cursor = Transaction().connection.cursor()
-        if (self.column_exist(old_name)
-                and not self.column_exist(new_name)):
-            cursor.execute('ALTER TABLE "%s" '
-                'RENAME COLUMN "%s" TO "%s"'
-                % (self.table_name, old_name, new_name))
-            self._update_definitions(columns=True)
-        elif exception and self.column_exist(new_name):
-            raise Exception('Unable to rename column %s.%s to %s.%s: '
-                '%s.%s already exist!'
-                % (self.table_name, old_name, self.table_name, new_name,
-                    self.table_name, new_name))
+        if self.column_exist(old_name):
+            if not self.column_exist(new_name):
+                cursor.execute(SQL(
+                        'ALTER TABLE {} RENAME COLUMN {} TO {}').format(
+                        Identifier(self.table_name),
+                        Identifier(old_name),
+                        Identifier(new_name)))
+                self._update_definitions(columns=True)
+            else:
+                logger.warning(
+                    'Unable to rename column %s on table %s to %s.',
+                    old_name, self.table_name, new_name)
 
     def _update_definitions(self,
             columns=None, constraints=None, indexes=None):
@@ -139,7 +155,7 @@ class TableHandler(TableHandlerInterface):
                 'FROM information_schema.columns '
                 'WHERE table_name = %s AND table_schema = %s',
                 (self.table_name, self.table_schema))
-            for column, typname, nullable, size, default in cursor.fetchall():
+            for column, typname, nullable, size, default in cursor:
                 self._columns[column] = {
                     'typname': typname,
                     'notnull': True if nullable == 'NO' else False,
@@ -153,7 +169,21 @@ class TableHandler(TableHandlerInterface):
                 'FROM information_schema.table_constraints '
                 'WHERE table_name = %s AND table_schema = %s',
                 (self.table_name, self.table_schema))
-            self._constraints = [c for c, in cursor.fetchall()]
+            self._constraints = [c for c, in cursor]
+
+            # add nonstandard exclude constraint
+            cursor.execute('SELECT c.conname '
+                'FROM pg_namespace nc, '
+                    'pg_namespace nr, '
+                    'pg_constraint c, '
+                    'pg_class r '
+                'WHERE nc.oid = c.connamespace AND nr.oid = r.relnamespace '
+                    'AND c.conrelid = r.oid '
+                    "AND c.contype = 'x' "  # exclude type
+                    "AND r.relkind IN ('r', 'p') "
+                    'AND r.relname = %s AND nr.nspname = %s',
+                    (self.table_name, self.table_schema))
+            self._constraints.extend((c for c, in cursor))
 
             cursor.execute('SELECT k.column_name, r.delete_rule '
                 'FROM information_schema.key_column_usage AS k '
@@ -162,7 +192,7 @@ class TableHandler(TableHandlerInterface):
                 'AND r.constraint_name = k.constraint_name '
                 'WHERE k.table_name = %s AND k.table_schema = %s',
                 (self.table_name, self.table_schema))
-            self._fk_deltypes = dict(cursor.fetchall())
+            self._fk_deltypes = dict(cursor)
 
         if indexes:
             # Fetch indexes defined for the table
@@ -173,7 +203,7 @@ class TableHandler(TableHandlerInterface):
                     "JOIN pg_class cl2 on (cl2.oid = ind.indexrelid) "
                 "WHERE cl.relname = %s AND n.nspname = %s",
                 (self.table_name, self.table_schema))
-            self._indexes = [l[0] for l in cursor.fetchall()]
+            self._indexes = [l[0] for l in cursor]
 
     @property
     def _field2module(self):
@@ -187,26 +217,33 @@ class TableHandler(TableHandlerInterface):
 
     def alter_size(self, column_name, column_type):
         cursor = Transaction().connection.cursor()
-        cursor.execute("ALTER TABLE \"%s\" "
-            "RENAME COLUMN \"%s\" TO _temp_change_size"
-            % (self.table_name, column_name))
-        cursor.execute("ALTER TABLE \"%s\" "
-            "ADD COLUMN \"%s\" %s"
-            % (self.table_name, column_name, column_type))
-        cursor.execute("UPDATE \"%s\" "
-            "SET \"%s\" = _temp_change_size::%s"
-            % (self.table_name, column_name, column_type))
-        cursor.execute("ALTER TABLE \"%s\" "
-            "DROP COLUMN _temp_change_size"
-            % (self.table_name,))
+        cursor.execute(
+            SQL("ALTER TABLE {} ALTER COLUMN {} TYPE {}").format(
+                Identifier(self.table_name),
+                Identifier(column_name),
+                SQL(column_type)))
         self._update_definitions(columns=True)
 
     def alter_type(self, column_name, column_type):
         cursor = Transaction().connection.cursor()
-        cursor.execute('ALTER TABLE "' + self.table_name + '" '
-            'ALTER "' + column_name + '" TYPE ' + column_type)
+        cursor.execute(SQL('ALTER TABLE {} ALTER {} TYPE {}').format(
+                Identifier(self.table_name),
+                Identifier(column_name),
+                SQL(column_type)))
         self._update_definitions(columns=True)
 
+    def column_is_type(self, column_name, type_, *, size=-1):
+        db_type = self._columns[column_name]['typname'].upper()
+
+        database = Transaction().database
+        base_type = database.sql_type(type_).base.upper()
+        if base_type == 'VARCHAR' and (size is None or size >= 0):
+            same_size = self._columns[column_name]['size'] == size
+        else:
+            same_size = True
+
+        return base_type == db_type and same_size
+
     def db_default(self, column_name, value):
         if value in [True, False]:
             test = str(value).lower()
@@ -214,8 +251,11 @@ class TableHandler(TableHandlerInterface):
             test = value
         if self._columns[column_name]['default'] != test:
             cursor = Transaction().connection.cursor()
-            cursor.execute('ALTER TABLE "' + self.table_name + '" '
-                'ALTER COLUMN "' + column_name + '" SET DEFAULT %s',
+            cursor.execute(
+                SQL(
+                    'ALTER TABLE {} ALTER COLUMN {} SET DEFAULT %s').format(
+                    Identifier(self.table_name),
+                    Identifier(column_name)),
                 (value,))
 
     def add_column(self, column_name, sql_type, default=None, comment=''):
@@ -228,14 +268,22 @@ class TableHandler(TableHandlerInterface):
 
         def add_comment():
             if comment and self.is_owner:
-                cursor.execute('COMMENT ON COLUMN "%s"."%s" IS \'%s\'' %
-                    (self.table_name, column_name, comment.replace("'", "''")))
+                cursor.execute(
+                    SQL('COMMENT ON COLUMN {}.{} IS %s').format(
+                        Identifier(self.table_name),
+                        Identifier(column_name)),
+                    (comment,))
         if self.column_exist(column_name):
             if (column_name in ('create_date', 'write_date')
                     and column_type[1].lower() != 'timestamp(6)'):
                 # Migrate dates from timestamp(0) to timestamp
-                cursor.execute('ALTER TABLE "' + self.table_name + '" '
-                    'ALTER COLUMN "' + column_name + '" TYPE timestamp')
+                cursor.execute(
+                    SQL(
+                        'ALTER TABLE {} ALTER COLUMN {} TYPE timestamp')
+                    .format(
+                        Identifier(self.table_name),
+                        Identifier(column_name)))
+
             add_comment()
             base_type = column_type[0].lower()
             if base_type != self._columns[column_name]['typname']:
@@ -274,8 +322,11 @@ class TableHandler(TableHandlerInterface):
             return
 
         column_type = column_type[1]
-        cursor.execute('ALTER TABLE "%s" ADD COLUMN "%s" %s'
-            % (self.table_name, column_name, column_type))
+        cursor.execute(
+            SQL('ALTER TABLE {} ADD COLUMN {} {}').format(
+                Identifier(self.table_name),
+                Identifier(column_name),
+                SQL(column_type)))
         add_comment()
 
         if default:
@@ -283,8 +334,11 @@ class TableHandler(TableHandlerInterface):
             cursor.execute('SELECT 1 FROM "%s" limit 1' % self.table_name)
             if cursor.rowcount:
                 # Populate column with default values:
-                cursor.execute('UPDATE "' + self.table_name + '" '
-                    'SET "' + column_name + '" = %s', (default(),))
+                cursor.execute(
+                    SQL('UPDATE {} SET {} = %s').format(
+                        Identifier(self.table_name),
+                        Identifier(column_name)),
+                    (default(),))
 
         self._update_definitions(columns=True)
 
@@ -305,39 +359,77 @@ class TableHandler(TableHandlerInterface):
         else:
             add = True
         if add:
-            cursor.execute('ALTER TABLE "' + self.table_name + '" '
-                'ADD CONSTRAINT "' + name + '" '
-                'FOREIGN KEY ("' + column_name + '") '
-                'REFERENCES "' + reference + '" '
-                'ON DELETE ' + on_delete)
+            cursor.execute(
+                SQL(
+                    "ALTER TABLE {table} "
+                    "ADD CONSTRAINT {constraint} "
+                    "FOREIGN KEY ({column}) REFERENCES {reference} "
+                    "ON DELETE {action}"
+                    )
+                .format(
+                    table=Identifier(self.table_name),
+                    constraint=Identifier(name),
+                    column=Identifier(column_name),
+                    reference=Identifier(reference),
+                    action=SQL(on_delete)))
         self._update_definitions(constraints=True)
 
     def drop_fk(self, column_name, table=None):
         self.drop_constraint(column_name + '_fkey', table=table)
 
-    def index_action(self, column_name, action='add', table=None):
-        if isinstance(column_name, basestring):
-            column_name = [column_name]
-        index_name = self.convert_name(
-            ((table or self.table_name) + "_" + '_'.join(column_name) +
-                "_index"))
+    def index_action(self, columns, action='add', where='', table=None):
+        if isinstance(columns, str):
+            columns = [columns]
+
+        def stringify(column):
+            if isinstance(column, str):
+                return column
+            else:
+                return ('_'.join(
+                        map(str, (column,) + column.params))
+                    .replace('"', '')
+                    .replace('%s', '__'))
+
+        name = [table or self.table_name]
+        name.append('_'.join(map(stringify, columns)))
+        if where:
+            name.append('+where')
+            name.append(stringify(where))
+        name.append('index')
+        index_name = self.convert_name('_'.join(name))
 
         with Transaction().connection.cursor() as cursor:
             if action == 'add':
                 if index_name in self._indexes:
                     return
-                cursor.execute('CREATE INDEX "' + index_name + '" '
-                    'ON "' + self.table_name + '" ( '
-                        + ','.join(['"' + x + '"' for x in column_name]) + ')')
+                params = sum(
+                    (c.params for c in columns if hasattr(c, 'params')), ())
+                if where:
+                    params += where.params
+                    where = ' WHERE %s' % where
+                cursor.execute(Composed([
+                            SQL('CREATE INDEX {name} ON {table} ({columns})')
+                            .format(
+                                name=Identifier(index_name),
+                                table=Identifier(self.table_name),
+                                columns=SQL(',').join(
+                                    (Identifier(c) if isinstance(c, str)
+                                        else SQL(str(c)))
+                                    for c in columns
+                                    )),
+                            SQL(where)
+                            ]),
+                    params)
                 self._update_definitions(indexes=True)
             elif action == 'remove':
-                if len(column_name) == 1:
-                    if (self._field2module.get(column_name[0],
-                                self.module_name) != self.module_name):
+                if len(columns) == 1 and isinstance(columns[0], str):
+                    if self._field2module.get(columns[0],
+                            self.module_name) != self.module_name:
                         return
 
                 if index_name in self._indexes:
-                    cursor.execute('DROP INDEX "%s" ' % (index_name,))
+                    cursor.execute(SQL('DROP INDEX {}').format(
+                            Identifier(index_name)))
                     self._update_definitions(indexes=True)
             else:
                 raise Exception('Index action not supported!')
@@ -350,100 +442,79 @@ class TableHandler(TableHandlerInterface):
             if action == 'add':
                 if self._columns[column_name]['notnull']:
                     return
-                cursor.execute('SELECT id FROM "%s" '
-                    'WHERE "%s" IS NULL'
-                    % (self.table_name, column_name))
+                cursor.execute(SQL(
+                        'SELECT id FROM {} WHERE {} IS NULL').format(
+                        Identifier(self.table_name),
+                        Identifier(column_name)))
                 if not cursor.rowcount:
-                    cursor.execute('ALTER TABLE "' + self.table_name + '" '
-                        'ALTER COLUMN "' + column_name + '" SET NOT NULL')
+                    cursor.execute(
+                        SQL(
+                            'ALTER TABLE {} ALTER COLUMN {} SET NOT NULL')
+                        .format(
+                            Identifier(self.table_name),
+                            Identifier(column_name)))
                     self._update_definitions(columns=True)
                 else:
                     logger.warning(
-                        'Unable to set column %s '
-                        'of table %s not null !\n'
-                        'Try to re-run: '
-                        'trytond.py --update=module\n'
-                        'If it doesn\'t work, update records '
-                        'and execute manually:\n'
-                        'ALTER TABLE "%s" ALTER COLUMN "%s" SET NOT NULL',
-                        column_name, self.table_name, self.table_name,
-                        column_name)
+                        "Unable to set not null on column %s of table %s.\n"
+                        "Try restarting one more time.\n"
+                        "If that doesn't work update the records and restart "
+                        "again.",
+                        column_name, self.table_name)
             elif action == 'remove':
                 if not self._columns[column_name]['notnull']:
                     return
                 if (self._field2module.get(column_name, self.module_name)
                         != self.module_name):
                     return
-                cursor.execute('ALTER TABLE "%s" '
-                    'ALTER COLUMN "%s" DROP NOT NULL'
-                    % (self.table_name, column_name))
+                cursor.execute(
+                    SQL('ALTER TABLE {} ALTER COLUMN {} DROP NOT NULL')
+                    .format(
+                        Identifier(self.table_name),
+                        Identifier(column_name)))
                 self._update_definitions(columns=True)
             else:
                 raise Exception('Not null action not supported!')
 
-    def add_constraint(self, ident, constraint, exception=False):
+    def add_constraint(self, ident, constraint):
         ident = self.convert_name(self.table_name + "_" + ident)
         if ident in self._constraints:
             # This constrain already exist
             return
         cursor = Transaction().connection.cursor()
-        try:
-            cursor.execute('ALTER TABLE "%s" '
-                'ADD CONSTRAINT "%s" %s'
-                % (self.table_name, ident, constraint), constraint.params)
-        except Exception:
-            if exception:
-                raise
-            logger.warning(
-                'unable to add \'%s\' constraint on table %s !\n'
-                'If you want to have it, you should update the records '
-                'and execute manually:\n'
-                'ALTER table "%s" ADD CONSTRAINT "%s" %s',
-                constraint, self.table_name, self.table_name, ident,
-                constraint)
+        cursor.execute(
+            SQL('ALTER TABLE {} ADD CONSTRAINT {} {}').format(
+                Identifier(self.table_name),
+                Identifier(ident),
+                SQL(str(constraint))),
+            constraint.params)
         self._update_definitions(constraints=True)
 
-    def drop_constraint(self, ident, exception=False, table=None):
+    def drop_constraint(self, ident, table=None):
         ident = self.convert_name((table or self.table_name) + "_" + ident)
         if ident not in self._constraints:
             return
         cursor = Transaction().connection.cursor()
-        try:
-            cursor.execute('ALTER TABLE "%s" '
-                'DROP CONSTRAINT "%s"'
-                % (self.table_name, ident))
-        except Exception:
-            if exception:
-                raise
-            logger.warning(
-                'unable to drop \'%s\' constraint on table %s!',
-                ident, self.table_name)
+        cursor.execute(
+            SQL('ALTER TABLE {} DROP CONSTRAINT {}').format(
+                Identifier(self.table_name), Identifier(ident)))
         self._update_definitions(constraints=True)
 
-    def drop_column(self, column_name, exception=False):
+    def drop_column(self, column_name):
         if not self.column_exist(column_name):
             return
         cursor = Transaction().connection.cursor()
-        try:
-            cursor.execute(
-                'ALTER TABLE "%s" DROP COLUMN "%s"' %
-                (self.table_name, column_name))
-
-        except Exception:
-            if exception:
-                raise
-            logger.warning(
-                'unable to drop \'%s\' column on table %s!',
-                column_name, self.table_name, exc_info=True)
+        cursor.execute(SQL('ALTER TABLE {} DROP COLUMN {}').format(
+                Identifier(self.table_name),
+                Identifier(column_name)))
         self._update_definitions(columns=True)
 
     @staticmethod
     def drop_table(model, table, cascade=False):
         cursor = Transaction().connection.cursor()
-        cursor.execute('DELETE FROM ir_model_data '
-            'WHERE model = \'%s\'' % model)
+        cursor.execute('DELETE FROM ir_model_data WHERE model = %s', (model,))
 
-        query = 'DROP TABLE "%s"' % table
+        query = 'DROP TABLE {}'
         if cascade:
             query = query + ' CASCADE'
-        cursor.execute(query)
+        cursor.execute(SQL(query).format(Identifier(table)))
