@@ -9,15 +9,15 @@ import re
 from collections import defaultdict
 from decimal import Decimal
 
-from . import __version__
-from .tools import grouped_slice
-from .transaction import Transaction
-from .pyson import PYSONEncoder, CONTEXT
+from trytond import __version__
+from trytond.pyson import PYSONEncoder, CONTEXT
+from trytond.tools import grouped_slice
+from trytond.transaction import Transaction
 
 logger = logging.getLogger(__name__)
 
-CDATA_START = re.compile('^\s*\<\!\[cdata\[', re.IGNORECASE)
-CDATA_END = re.compile('\]\]\>\s*$', re.IGNORECASE)
+CDATA_START = re.compile(r'^\s*\<\!\[cdata\[', re.IGNORECASE)
+CDATA_END = re.compile(r'\]\]\>\s*$', re.IGNORECASE)
 
 
 class DummyTagHandler:
@@ -221,28 +221,29 @@ class RecordTagHandler:
             eval_attr = attributes.get('eval', '')
             pyson_attr = bool(int(attributes.get('pyson', '0')))
 
+            context = {}
+            context['time'] = time
+            context['version'] = __version__.rsplit('.', 1)[0]
+            context['ref'] = self.mh.get_id
+            context['Decimal'] = Decimal
+            context['datetime'] = datetime
+            if pyson_attr:
+                context.update(CONTEXT)
+
             if search_attr:
                 search_model = self.model._fields[field_name].model_name
                 SearchModel = self.mh.pool.get(search_model)
                 with Transaction().set_context(active_test=False):
-                    found, = SearchModel.search(eval(search_attr))
+                    found, = SearchModel.search(eval(search_attr, context))
                     self.values[field_name] = found.id
 
             elif ref_attr:
                 self.values[field_name] = self.mh.get_id(ref_attr)
 
             elif eval_attr:
-                context = {}
-                context['time'] = time
-                context['version'] = __version__.rsplit('.', 1)[0]
-                context['ref'] = self.mh.get_id
-                context['Decimal'] = Decimal
-                context['datetime'] = datetime
-                if pyson_attr:
-                    context.update(CONTEXT)
                 value = eval(eval_attr, context)
                 if pyson_attr:
-                    value = PYSONEncoder().encode(value)
+                    value = PYSONEncoder(sort_keys=True).encode(value)
                 self.values[field_name] = value
 
         else:
@@ -279,7 +280,7 @@ class RecordTagHandler:
                 self.cdata = 'done'
 
                 value = self.values[self.current_field]
-                match = re.findall('[^%]%\((.*?)\)[ds]', value)
+                match = re.findall(r'[^%]%\((.*?)\)[ds]', value)
                 xml_ids = {}
                 for xml_id in match:
                     xml_ids[xml_id] = self.mh.get_id(xml_id)
@@ -330,6 +331,11 @@ class Fs2bdAccessor:
             self.fetch_new_module(module)
         return self.fs2db[module].get(fs_id, None)
 
+    def exists(self, module, fs_id):
+        if module not in self.fetched_modules:
+            self.fetch_new_module(module)
+        return fs_id in self.fs2db[module]
+
     def get_browserecord(self, module, model_name, db_id):
         if module not in self.fetched_modules:
             self.fetch_new_module(module)
@@ -366,7 +372,7 @@ class Fs2bdAccessor:
                             cache, cache.get('_language_cache', {}).values()):
                         if (model_name in cache
                                 and model.id in cache[model_name]):
-                            cache[model_name][model.id] = {}
+                            del cache[model_name][model.id]
             self.browserecord[module][model_name][model.id] = model
 
     def fetch_new_module(self, module):
@@ -405,7 +411,7 @@ class Fs2bdAccessor:
 
 class TrytondXmlHandler(sax.handler.ContentHandler):
 
-    def __init__(self, pool, module, module_state, modules):
+    def __init__(self, pool, module, module_state, modules, languages):
         "Register known taghandlers, and managed tags."
         sax.handler.ContentHandler.__init__(self)
 
@@ -422,6 +428,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
         self.grouped_model_data = []
         self.skip_data = False
         self.modules = modules
+        self.languages = languages
 
         # Tag handlders are used to delegate the processing
         self.taghandlerlist = {
@@ -449,11 +456,8 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
 
         try:
             self.sax_parser.parse(source)
-        except Exception:
-            logger.error(
-                "Error while parsing xml file:\n" + self.current_state(),
-                exc_info=True)
-            raise
+        except Exception as e:
+            raise Exception("Error " + self.current_state()) from e
         return self.to_delete
 
     def startElement(self, name, attributes):
@@ -467,15 +471,14 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
             elif name == "data":
                 self.noupdate = bool(int(attributes.get("noupdate", '0')))
                 self.grouped = bool(int(attributes.get('grouped', 0)))
-                if self.pool.test and \
-                        bool(int(attributes.get("skiptest", '0'))):
-                    self.skip_data = True
-                else:
-                    self.skip_data = False
+                self.skip_data = False
                 depends = attributes.get('depends', '').split(',')
                 depends = {m.strip() for m in depends if m}
                 if not depends.issubset(self.modules):
                     self.skip_data = True
+                if (attributes.get('language')
+                        and attributes.get('language') not in self.languages):
+                    self.skip_data = True
 
             elif name == "tryton":
                 pass
@@ -583,7 +586,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
 
         Model = self.pool.get(model)
 
-        if self.fs2db.get(module, fs_id):
+        if self.fs2db.exists(module, fs_id):
 
             # Remove this record from the to_delete list. This means that
             # the corresponding record have been found.
@@ -599,6 +602,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 self.fs2db.get(module, fs_id)[x]
                 for x in ["db_id", "model", "id", "values"]]
 
+            # Check if record has not been deleted
+            if db_id is None:
+                return
+
             if not old_values:
                 old_values = {}
             else:
@@ -755,7 +762,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
             fs_values = old_values.copy()
             fs_values.update(new_values)
 
-            if values != old_values:
+            if values != fs_values:
                 self.grouped_model_data.extend(([self.ModelData(mdata_id)], {
                             'fs_id': fs_id,
                             'model': model,
@@ -763,6 +770,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                             'db_id': record.id,
                             'values': self.ModelData.dump_values(values),
                             'fs_values': self.ModelData.dump_values(fs_values),
+                            'noupdate': self.noupdate,
                             }))
 
         # reset_browsercord to keep cache memory low
@@ -800,15 +808,12 @@ def post_import(pool, module, to_delete):
                 logger.warning(
                     'Could not delete id %d of model %s because model no '
                     'longer exists.', db_id, model)
-            transaction.commit()
         except Exception:
             transaction.rollback()
             logger.error(
-                'Could not delete id: %d of model %s\n'
-                'There should be some relation '
-                'that points to this resource\n'
-                'You should manually fix this '
-                'and restart --update=module\n',
+                "Could not delete id %d from model %s.\n"
+                "There may be a relation that points to this resource "
+                "that must be manually fixed before restarting the update.",
                 db_id, model, exc_info=True)
             if 'active' in Model._fields:
                 try:
@@ -820,6 +825,7 @@ def post_import(pool, module, to_delete):
                     logger.error(
                         'Could not inactivate id: %d of model %s\n',
                         db_id, model, exc_info=True)
+        transaction.commit()
 
     # Clean model_data:
     if mdata_delete:
