@@ -4,17 +4,22 @@ from .model import Model
 from .modelview import ModelView
 from .modelstorage import ModelStorage, EvalEnvironment
 from .modelsingleton import ModelSingleton
-from .modelsql import ModelSQL, Check, Unique
+from .modelsql import ModelSQL, Check, Unique, Exclude
 from .workflow import Workflow
 from .dictschema import DictSchemaMixin
 from .match import MatchMixin
 from .union import UnionMixin
+from .symbol import SymbolMixin
 from .multivalue import MultiValueMixin, ValueMixin
 from .descriptors import dualmethod
 from .order import sequence_ordered
+from .active import DeactivableMixin
+from .tree import tree
+from .avatar import avatar_mixin
 
 __all__ = ['Model', 'ModelView', 'ModelStorage', 'ModelSingleton', 'ModelSQL',
-    'Check', 'Unique',
+    'Check', 'Unique', 'Exclude',
     'Workflow', 'DictSchemaMixin', 'MatchMixin', 'UnionMixin', 'dualmethod',
-    'MultiValueMixin', 'ValueMixin',
-    'EvalEnvironment', 'sequence_ordered']
+    'MultiValueMixin', 'ValueMixin', 'SymbolMixin',
+    'EvalEnvironment', 'sequence_ordered', 'DeactivableMixin', 'tree',
+    'avatar_mixin']
