@@ -3,21 +3,23 @@
 import warnings
 from functools import wraps
 
+import sql
 from sql import (operators, Column, Literal, Select, CombiningQuery, Null,
     Query, Expression, Cast)
+from sql.aggregate import Min
 from sql.conditionals import Coalesce, NullIf
 from sql.operators import Concat
 
 from trytond import backend
-from trytond.pyson import PYSON, PYSONEncoder, Eval
-from trytond.const import OPERATORS
-from trytond.transaction import Transaction
-from trytond.pool import Pool
 from trytond.cache import LRUDictTransaction
+from trytond.const import OPERATORS
+from trytond.pool import Pool
+from trytond.pyson import PYSON, PYSONEncoder, PYSONDecoder, Eval
+from trytond.rpc import RPC
+from trytond.tools.string_ import StringPartitioned, LazyString
+from trytond.transaction import Transaction
 
-from ...rpc import RPC
-
-Database = backend.get('Database')
+_sql_version = tuple(map(int, sql.__version__.split('.')))
 
 
 def domain_validate(value):
@@ -31,9 +33,11 @@ def domain_validate(value):
             elif (isinstance(arg, tuple)
                 or (isinstance(arg, list)
                     and len(arg) > 2
-                    and ((isinstance(arg[1], str)
+                    and ((
+                                isinstance(arg[1], str)
                                 and arg[1] in OPERATORS)
-                        or (isinstance(arg[1], PYSON)
+                        or (
+                                isinstance(arg[1], PYSON)
                                 and arg[1].types() == set([str]))))):
                 pass
             elif isinstance(arg, list):
@@ -45,9 +49,11 @@ def domain_validate(value):
 
 def states_validate(value):
     assert isinstance(value, dict), 'states must be a dict'
+    assert set(value).issubset({'required', 'readonly', 'invisible'}), (
+        'extra keys "%(keys)s" in states' % {
+            'keys': set(value) - {'required', 'readonly', 'invisible'},
+            })
     for state in value:
-        if state == 'icon':
-            continue
         assert isinstance(value[state], (bool, PYSON)), \
             'values of states must be PYSON'
         if hasattr(value[state], 'types'):
@@ -138,16 +144,19 @@ def get_eval_fields(value):
     return encoder.fields
 
 
-def instanciate_values(Target, value):
+def instanciate_values(Target, value, **extra):
     from ..modelstorage import ModelStorage, cache_size
     kwargs = {}
     ids = []
     if issubclass(Target, ModelStorage):
-        kwargs['_local_cache'] = LRUDictTransaction(cache_size())
+        kwargs['_local_cache'] = LRUDictTransaction(
+            cache_size(), Target._record)
         kwargs['_ids'] = ids
 
     def instance(data):
         if isinstance(data, Target):
+            for k, v in extra.items():
+                setattr(data, k, v)
             return data
         elif isinstance(data, dict):
             if data.get('id', -1) >= 0:
@@ -157,13 +166,29 @@ def instanciate_values(Target, value):
                 ids.append(data['id'])
             else:
                 values = data
+            values.update(extra)
             return Target(**values)
         else:
             ids.append(data)
-            return Target(data, **kwargs)
+            return Target(data, **extra, **kwargs)
     return tuple(instance(x) for x in (value or []))
 
 
+def instantiate_context(field, record):
+    from ..modelstorage import EvalEnvironment
+    ctx = {}
+    if field.context:
+        pyson_context = PYSONEncoder().encode(field.context)
+        ctx.update(PYSONDecoder(
+                EvalEnvironment(record, record.__class__)).decode(
+                pyson_context))
+    datetime_ = None
+    if getattr(field, 'datetime_field', None):
+        datetime_ = getattr(record, field.datetime_field, None)
+        ctx = {'_datetime': datetime_}
+    return ctx
+
+
 def on_change_result(record):
     return record._changed_values
 
@@ -195,6 +220,7 @@ SQL_OPERATORS = {
 class Field(object):
     _type = None
     _sql_type = None
+    _py_type = None
 
     def __init__(self, string='', help='', required=False, readonly=False,
             domain=None, states=None, select=False, on_change=None,
@@ -225,7 +251,8 @@ class Field(object):
         :param loading: Define how the field must be loaded:
             ``lazy`` or ``eager``.
         '''
-        assert string, 'a string is required'
+        if not isinstance(string, LazyString):
+            assert string, 'a string is required'
         self.string = string
         self.help = help
         self.required = required
@@ -256,6 +283,22 @@ class Field(object):
         self.loading = loading
         self.name = None
 
+    @property
+    def string(self):
+        return self.__string
+
+    @string.setter
+    def string(self, value):
+        self.__string = StringPartitioned(value)
+
+    @property
+    def help(self):
+        return self.__help
+
+    @help.setter
+    def help(self, value):
+        self.__help = StringPartitioned(value)
+
     def _get_domain(self):
         return self.__domain
 
@@ -303,7 +346,7 @@ class Field(object):
     def __set__(self, inst, value):
         assert self.name is not None
         if inst._values is None:
-            inst._values = {}
+            inst._values = inst._record()
         inst._values[self.name] = value
 
     def sql_format(self, value):
@@ -312,6 +355,9 @@ class Field(object):
 
         assert self._sql_type is not None
         database = Transaction().database
+        if (self._py_type and value is not None
+                and not isinstance(value, self._py_type)):
+            value = self._py_type(value)
         return database.sql_format(self._sql_type, value)
 
     def sql_type(self):
@@ -387,62 +433,194 @@ class Field(object):
             model.__rpc__.setdefault(
                 func_name, RPC(instantiate=0, result=result))
 
+    def definition(self, model, language):
+        pool = Pool()
+        Translation = pool.get('ir.translation')
+        encoder = PYSONEncoder()
+        definition = {
+            'context': encoder.encode(self.context),
+            'loading': self.loading,
+            'name': self.name,
+            'depends': self.depends,
+            'on_change': list(self.on_change),
+            'on_change_with': list(self.on_change_with),
+            'readonly': self.readonly,
+            'required': self.required,
+            'states': encoder.encode(self.states),
+            'type': self._type,
+            'domain': encoder.encode(self.domain),
+            'searchable': hasattr(model, 'search'),
+            'sortable': hasattr(model, 'search'),
+            }
+
+        # Add id to on_change's if they are not cached
+        # Not having the id increase the efficiency of the cache
+        for method in ['on_change', 'on_change_with']:
+            changes = definition[method]
+            if changes:
+                method_name = method + '_' + self.name
+                if not model.__rpc__[method_name].cache:
+                    changes.append('id')
+
+                for name in changes:
+                    target = model
+                    if '.' in name:
+                        prefix, _ = name.rsplit('.', 1)
+                        prefix += '.'
+                    else:
+                        prefix = ''
+                    while name.startswith('_parent_'):
+                        field, name = name.split('.', 1)
+                        target = target._fields[field[8:]].get_target()
+                    field = target._fields[name]
+                    if field and field.context:
+                        eval_fields = get_eval_fields(field.context)
+                        for context_field_name in eval_fields:
+                            prefix_ctx_field_name = (
+                                prefix + context_field_name)
+                            if (context_field_name in field.depends
+                                    and prefix_ctx_field_name not in changes):
+                                changes.append(prefix_ctx_field_name)
+
+        name = '%s,%s' % (model.__name__, self.name)
+        for attr, ttype in [('string', 'field'), ('help', 'help')]:
+            definition[attr] = ''
+            for source in getattr(self, attr):
+                if not isinstance(source, LazyString):
+                    source = (
+                        Translation.get_source(name, ttype, language, source)
+                        or source)
+                definition[attr] += source
+        return definition
+
+    def definition_translations(self, model, language):
+        "Returns sources used for definition"
+        name = '%s,%s' % (model.__name__, self.name)
+        translations = []
+        for attr, ttype in [('string', 'field'), ('help', 'help')]:
+            for source in getattr(self, attr):
+                if not isinstance(source, LazyString):
+                    translations.append((name, ttype, language, source))
+        return translations
+
 
 class FieldTranslate(Field):
 
     def _get_translation_join(self, Model, name,
             translation, model, table, from_, language):
-        if Model.__name__ == 'ir.model':
-            return from_.join(translation, 'LEFT',
-                condition=(translation.name == Concat(Concat(
-                            table.model, ','), name))
-                & (translation.res_id == -1)
-                & (translation.lang == language)
-                & (translation.type == 'model')
-                & (translation.fuzzy == False))
-        elif Model.__name__ == 'ir.model.field':
+        if Model.__name__ == 'ir.model.field':
+            pool = Pool()
+            IrModel = pool.get('ir.model')
+            ModelData = pool.get('ir.model.data')
+            ModelField = pool.get('ir.model.field')
+            Translation = pool.get('ir.translation')
+            model = IrModel.__table__()
+            model_data = ModelData.__table__()
+            model_field = ModelField.__table__()
+            msg_trans = Translation.__table__()
             if name == 'field_description':
                 type_ = 'field'
             else:
                 type_ = 'help'
-            return from_.join(model, 'LEFT',
-                condition=model.id == table.model).join(
-                    translation, 'LEFT',
-                    condition=(translation.name == Concat(Concat(
-                                model.model, ','), table.name))
-                    & (translation.res_id == -1)
-                    & (translation.lang == language)
-                    & (translation.type == type_)
-                    & (translation.fuzzy == False))
+            translation = translation.select(
+                translation.id.as_('id'),
+                translation.res_id.as_('res_id'),
+                translation.value.as_('value'),
+                translation.name.as_('name'),
+                translation.lang.as_('lang'),
+                translation.type.as_('type'),
+                translation.fuzzy.as_('fuzzy'),
+                )
+            translation |= (msg_trans
+                .join(model_data,
+                    condition=(msg_trans.res_id == model_data.db_id)
+                    & (model_data.model == 'ir.message')
+                    & (msg_trans.name == 'ir.message,text'))
+                .join(model_field,
+                    condition=Concat(
+                        Concat(model_data.module, '.'),
+                        model_data.fs_id) == getattr(model_field, name))
+                .join(model,
+                    condition=model_field.model == model.id)
+                .select(
+                    msg_trans.id.as_('id'),
+                    Literal(-1).as_('res_id'),
+                    msg_trans.value.as_('value'),
+                    Concat(
+                        Concat(model.model, ','),
+                        model_field.name).as_('name'),
+                    msg_trans.lang.as_('lang'),
+                    Literal(type_).as_('type'),
+                    msg_trans.fuzzy.as_('fuzzy'),
+                    ))
+        if backend.name == 'postgresql' and _sql_version >= (1, 1, 0):
+            query = translation.select(
+                translation.res_id.as_('res_id'),
+                translation.value.as_('value'),
+                translation.name.as_('name'),
+                distinct=True,
+                distinct_on=[translation.res_id, translation.name],
+                order_by=[
+                    translation.res_id,
+                    translation.name,
+                    translation.id.desc])
         else:
-            return from_.join(translation, 'LEFT',
-                condition=(translation.res_id == table.id)
-                & (translation.name == '%s,%s' % (Model.__name__, name))
-                & (translation.lang == language)
-                & (translation.type == 'model')
-                & (translation.fuzzy == False))
+            query = translation.select(
+                translation.res_id.as_('res_id'),
+                Min(translation.value).as_('value'),
+                translation.name.as_('name'),
+                group_by=[translation.res_id, translation.name])
+        if Model.__name__ == 'ir.model':
+            name_ = Concat(Concat(table.model, ','), name)
+            type_ = 'model'
+            res_id = -1
+        elif Model.__name__ == 'ir.model.field':
+            from_ = from_.join(model, 'LEFT',
+                condition=model.id == table.model)
+            name_ = Concat(Concat(model.model, ','), table.name)
+            if name == 'field_description':
+                type_ = 'field'
+            else:
+                type_ = 'help'
+            res_id = -1
+        else:
+            name_ = '%s,%s' % (Model.__name__, name)
+            type_ = 'model'
+            res_id = table.id
+        query.where = (
+            (translation.lang == language)
+            & (translation.type == type_)
+            & (translation.fuzzy == Literal(False))
+            )
+        return query, from_.join(query, 'LEFT',
+            condition=(query.res_id == res_id) & (query.name == name_))
 
-    def convert_domain(self, domain, tables, Model):
+    def _get_translation_column(self, Model, name):
         from trytond.ir.lang import get_parent_language
         pool = Pool()
         Translation = pool.get('ir.translation')
         IrModel = pool.get('ir.model')
-        if not self.translate:
-            return super(FieldTranslate, self).convert_domain(
-                domain, tables, Model)
 
         table = join = Model.__table__()
         model = IrModel.__table__()
-        name, operator, value = domain
         language = Transaction().language
         column = None
         while language:
             translation = Translation.__table__()
-            join = self._get_translation_join(
+            translation, join = self._get_translation_join(
                 Model, name, translation, model, table, join, language)
             column = Coalesce(NullIf(column, ''), translation.value)
             language = get_parent_language(language)
-        column = Coalesce(NullIf(column, ''), self.sql_column(table))
+        return table, join, column
+
+    def convert_domain(self, domain, tables, Model):
+        if not self.translate:
+            return super(FieldTranslate, self).convert_domain(
+                domain, tables, Model)
+        table, _ = tables[None]
+        name, operator, value = domain
+        model, join, column = self._get_translation_column(Model, name)
+        column = Coalesce(NullIf(column, ''), self.sql_column(model))
         column = self._domain_column(operator, column)
         Operator = SQL_OPERATORS[operator]
         assert name == self.name
@@ -452,20 +630,14 @@ class FieldTranslate(Field):
         elif isinstance(where, operators.NotIn) and not where.right:
             where = Literal(True)
         where = self._domain_add_null(column, operator, value, where)
-        return tables[None][0].id.in_(join.select(table.id, where=where))
+        return table.id.in_(join.select(model.id, where=where))
 
-    def convert_order(self, name, tables, Model):
+    def _get_translation_order(self, tables, Model, name):
         from trytond.ir.lang import get_parent_language
         pool = Pool()
         Translation = pool.get('ir.translation')
         IrModel = pool.get('ir.model')
-        if not self.translate:
-            return super(FieldTranslate, self).convert_order(name, tables,
-                Model)
-        assert name == self.name
-
         table, _ = tables[None]
-
         join = table
         language = Transaction().language
         column = None
@@ -474,7 +646,7 @@ class FieldTranslate(Field):
             if key not in tables:
                 translation = Translation.__table__()
                 model = IrModel.__table__()
-                join = self._get_translation_join(
+                translation, join = self._get_translation_join(
                     Model, name, translation, model, table, table, language)
                 if join.left == table:
                     tables[key] = {
@@ -494,5 +666,17 @@ class FieldTranslate(Field):
                     translation, _ = tables[key]['translation'][None]
             column = Coalesce(NullIf(column, ''), translation.value)
             language = get_parent_language(language)
+        return column
 
-        return [Coalesce(column, self.sql_column(table))]
+    def convert_order(self, name, tables, Model):
+        if not self.translate:
+            return super().convert_order(name, tables, Model)
+        assert name == self.name
+        table, _ = tables[None]
+        column = self._get_translation_order(tables, Model, name)
+        return [Coalesce(NullIf(column, ''), self.sql_column(table))]
+
+    def definition(self, model, language):
+        definition = super().definition(model, language)
+        definition['translate'] = self.translate
+        return definition
