@@ -5,11 +5,12 @@ from sql.aggregate import Max
 from sql.conditionals import Coalesce
 from sql.operators import Or
 
+from trytond.pool import Pool
+from trytond.pyson import PYSONEncoder
+from trytond.tools import reduce_ids
+from trytond.transaction import Transaction
 from .field import (Field, search_order_validate, context_validate,
-    with_inactive_records)
-from ...pool import Pool
-from ...tools import reduce_ids
-from ...transaction import Transaction
+    with_inactive_records, instantiate_context)
 
 
 class Many2One(Field):
@@ -18,6 +19,7 @@ class Many2One(Field):
     '''
     _type = 'many2one'
     _sql_type = 'INTEGER'
+    _py_type = int
 
     def __init__(self, model_name, string='', left=None, right=None,
             ondelete='SET NULL', datetime_field=None, target_search='join',
@@ -99,18 +101,19 @@ class Many2One(Field):
 
     def __set__(self, inst, value):
         Target = self.get_target()
-        if isinstance(value, dict):
-            value = Target(**value)
-        elif isinstance(value, int):
-            value = Target(value)
+        if isinstance(value, (dict, int)):
+            ctx = instantiate_context(self, inst)
+            with Transaction().set_context(ctx):
+                if isinstance(value, dict):
+                    value = Target(**value)
+                elif isinstance(value, int):
+                    value = Target(value)
         assert isinstance(value, (Target, type(None)))
         super(Many2One, self).__set__(inst, value)
 
     def sql_format(self, value):
-        if value is None:
-            return None
         assert value is not False
-        return int(value)
+        return super().sql_format(value)
 
     def convert_domain_mptt(self, domain, tables):
         cursor = Transaction().connection.cursor()
@@ -122,7 +125,7 @@ class Many2One(Field):
         right = getattr(Target, self.right).sql_column(table)
         cursor.execute(*table.select(left, right, where=red_sql))
         where = Or()
-        for l, r in cursor.fetchall():
+        for l, r in cursor:
             if operator.endswith('child_of'):
                 where.append((left >= l) & (right <= r))
             else:
@@ -289,3 +292,21 @@ class Many2One(Field):
                 }
             tables[self.name] = target_tables
         return target_tables
+
+    def definition(self, model, language):
+        encoder = PYSONEncoder()
+
+        target = self.get_target()
+        relation_fields = [fname for fname, field in target._fields.items()
+            if field._type == 'one2many'
+            and field.model_name == model.__name__
+            and field.field == self.name]
+
+        definition = super().definition(model, language)
+        definition['datetime_field'] = self.datetime_field
+        definition['relation'] = target.__name__
+        if len(relation_fields) == 1:
+            definition['relation_field'], = relation_fields
+        definition['search_context'] = encoder.encode(self.search_context)
+        definition['search_order'] = encoder.encode(self.search_order)
+        return definition
