@@ -743,7 +743,10 @@ class ModelSQL(ModelStorage):
                 if not callable(cls.table_query):
                     rule_domain = Rule.domain_get(
                         cls.__name__, mode=f.lstrip('_'))
-                    if rule_domain:
+                    # No need to compute rule domain if it is the same as the
+                    # read rule domain because it is already applied as where
+                    # clause.
+                    if rule_domain and rule_domain != domain:
                         rule_tables = {None: (table, None)}
                         rule_tables, rule_expression = cls.search_domain(
                             rule_domain, active_test=False, tables=rule_tables)
@@ -878,9 +881,13 @@ class ModelSQL(ModelStorage):
                 add = target_ids.extend
             elif field._type == 'reference':
                 def add(value):
-                    id_ = int(value.split(',', 1)[1])
-                    if id_ >= 0:
-                        target_ids.append(id_)
+                    try:
+                        id_ = int(value.split(',', 1)[1])
+                    except ValueError:
+                        pass
+                    else:
+                        if id_ >= 0:
+                            target_ids.append(id_)
             else:
                 add = target_ids.append
             for row in rows:
@@ -902,7 +909,10 @@ class ModelSQL(ModelStorage):
                 for row in rows:
                     value = row[name]
                     if isinstance(value, str):
-                        value = int(value.split(',', 1)[1])
+                        try:
+                            value = int(value.split(',', 1)[1])
+                        except ValueError:
+                            value = None
                     if value is not None and value >= 0:
                         row[key] = targets[value]
                     else:
