@@ -2,16 +2,16 @@
 # this repository contains the full copyright notices and license terms.
 from lxml import etree
 from functools import wraps
-import copy
 import collections
 
+from trytond.exceptions import UserError
+from trytond.i18n import gettext
 from trytond.model import Model, fields
 from trytond.tools import ClassProperty, is_instance_method
 from trytond.pyson import PYSONDecoder, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.cache import Cache
 from trytond.pool import Pool
-from trytond.exceptions import UserError
 from trytond.rpc import RPC
 
 from .fields import on_change_result
@@ -19,6 +19,10 @@ from .fields import on_change_result
 __all__ = ['ModelView']
 
 
+class AccessButtonError(UserError):
+    pass
+
+
 def _find(tree, element):
     if element.tag == 'xpath':
         res = tree.xpath(element.get('expr'))
@@ -92,6 +96,7 @@ class ModelView(Model):
     """
     Define a model with views in Tryton.
     """
+    __slots__ = ()
     __modules_list = None  # Cache for the modules list sorted by dependency
     _fields_view_get_cache = Cache('modelview.fields_view_get')
     _view_toolbar_get_cache = Cache('modelview.view_toolbar_get')
@@ -113,43 +118,49 @@ class ModelView(Model):
     @classmethod
     def __setup__(cls):
         super(ModelView, cls).__setup__()
-        cls.__rpc__['fields_view_get'] = RPC()
-        cls.__rpc__['view_toolbar_get'] = RPC()
+        cls.__rpc__['fields_view_get'] = RPC(cache=dict(days=1))
+        cls.__rpc__['view_toolbar_get'] = RPC(cache=dict(days=1))
         cls.__rpc__['on_change'] = RPC(instantiate=0)
         cls.__rpc__['on_change_with'] = RPC(instantiate=0)
         cls._buttons = {}
 
         fields_ = {}
-        callables = {}
         for name in dir(cls):
             if name.startswith('__'):
                 continue
             attr = getattr(cls, name)
             if isinstance(attr, fields.Field):
                 fields_[name] = attr
-            elif callable(attr):
-                callables[name] = attr
+
+    @classmethod
+    def __post_setup__(cls):
+        super(ModelView, cls).__post_setup__()
 
         methods = {
+            '_done': set(),
             'depends': collections.defaultdict(set),
             'depend_methods': collections.defaultdict(set),
             'change': collections.defaultdict(set),
             }
         cls.__change_buttons = methods['change']
 
-        def get_callable_attributes(name, method):
+        def set_methods(name):
+            if name in methods['_done']:
+                return
+            methods['_done'].add(name)
             for parent_cls in cls.__mro__:
                 parent_meth = getattr(parent_cls, name, None)
                 if not parent_meth:
                     continue
                 for attr in ['depends', 'depend_methods', 'change']:
-                    parent_value = getattr(parent_meth, attr, None)
+                    if isinstance(parent_meth, property):
+                        parent_value = getattr(parent_meth.fget, attr, set())
+                        parent_value |= getattr(parent_meth.fset, attr, set())
+                    else:
+                        parent_value = getattr(parent_meth, attr, set())
                     if parent_value:
                         methods[attr][name] |= parent_value
 
-        for name, method in callables.items():
-            get_callable_attributes(name, method)
-
         def setup_field(field_name, field, attribute):
             if attribute == 'selection_change_with':
                 if isinstance(
@@ -159,19 +170,21 @@ class ModelView(Model):
                     return
             else:
                 function_name = '%s_%s' % (attribute, field_name)
-            if not getattr(cls, function_name, None):
+            function = getattr(cls, function_name, None)
+            if not function:
                 return
 
-            function = getattr(cls, function_name, None)
-            setattr(field, attribute,
-                getattr(field, attribute) | methods['depends'][function_name])
+            set_methods(function_name)
+            setattr(field, attribute, methods['depends'][function_name])
 
             meth_names = list(methods['depend_methods'][function_name])
             meth_done = set()
             while meth_names:
                 meth_name = meth_names.pop()
-                assert callable(getattr(cls, meth_name)), \
-                    "%s.%s not callable" % (cls, meth_name)
+                method = getattr(cls, meth_name)
+                assert callable(method) or isinstance(method, property), \
+                    "%s.%s not callable or property" % (cls, meth_name)
+                set_methods(meth_name)
                 setattr(field, attribute,
                     getattr(field, attribute) | methods['depends'][meth_name])
                 meth_names += list(
@@ -183,7 +196,7 @@ class ModelView(Model):
                 # Decorate on_change to always return self
                 setattr(cls, function_name, on_change(function))
 
-        for name, field in fields_.items():
+        for name, field in cls._fields.items():
             for attribute in [
                     'on_change',
                     'on_change_with',
@@ -192,10 +205,6 @@ class ModelView(Model):
                     ]:
                 setup_field(name, field, attribute)
 
-    @classmethod
-    def __post_setup__(cls):
-        super(ModelView, cls).__post_setup__()
-
         # Update __rpc__
         for field_name, field in cls._fields.items():
             field.set_rpc(cls)
@@ -208,15 +217,28 @@ class ModelView(Model):
                 cls.__rpc__.setdefault(button,
                     RPC(instantiate=0, result=on_change_result))
 
+            meth_names = set()
+            meth_done = set()
             for parent_cls in cls.__mro__:
                 parent_meth = getattr(parent_cls, button, None)
                 if not parent_meth:
                     continue
                 cls.__change_buttons[button] |= getattr(
                     parent_meth, 'change', set())
+                meth_names |= getattr(parent_meth, 'change_methods', set())
+            while meth_names:
+                meth_name = meth_names.pop()
+                method = getattr(cls, meth_name)
+                assert callable(method) or isinstance(method, property), \
+                    "%s.%s not callable or property" % (cls, meth_name)
+                set_methods(meth_name)
+                cls.__change_buttons[button] |= methods['depends'][meth_name]
+                meth_names |= (
+                    methods['depend_methods'][meth_name] - meth_done)
+                meth_done.add(meth_name)
 
     @classmethod
-    def fields_view_get(cls, view_id=None, view_type='form'):
+    def fields_view_get(cls, view_id=None, view_type='form', level=None):
         '''
         Return a view definition.
         If view_id is None the first one will be used of view_type.
@@ -228,7 +250,7 @@ class ModelView(Model):
            - fields: a dictionary with the definition of each field in the view
            - field_childs: the name of the childs field for tree
         '''
-        key = (cls.__name__, view_id, view_type)
+        key = (cls.__name__, view_id, view_type, level)
         result = cls._fields_view_get_cache.get(key)
         if result:
             return result
@@ -294,7 +316,10 @@ class ModelView(Model):
                     # There is perhaps a new module in the directory
                     ModelView._reset_modules_list()
                     raise_p = True
-            parser = etree.XMLParser(remove_comments=True)
+            if not result['arch']:
+                raise ValueError("Missing view architecture for %s" % ((
+                            cls.__name__, view_id, view_type),))
+            parser = etree.XMLParser(remove_comments=True, resolve_entities=False)
             tree = etree.fromstring(result['arch'], parser=parser)
             for view in views:
                 if view.domain:
@@ -325,6 +350,11 @@ class ModelView(Model):
                             xml += "<newline/>"
                     else:
                         xml += '<field name="%s" colspan="4"/>' % (i,)
+                if cls._buttons:
+                    xml += '<group id="buttons" col="-1" colspan="4">'
+                    for button in cls._buttons:
+                        xml += '<button name="%s"/>' % button
+                    xml += '</group>'
                 xml += "</form>"
             elif view_type == 'tree':
                 field = 'id'
@@ -340,13 +370,15 @@ class ModelView(Model):
             result['field_childs'] = None
             result['view_id'] = view_id
 
+        if level is None:
+            level = 1 if result['type'] == 'tree' else 0
+
         # Update arch and compute fields from arch
-        parser = etree.XMLParser(remove_blank_text=True)
+        parser = etree.XMLParser(
+            remove_blank_text=True, resolve_entities=False)
         tree = etree.fromstring(result['arch'], parser)
-        xarch, xfields = cls._view_look_dom_arch(tree, result['type'],
-                result['field_childs'])
-        result['arch'] = xarch
-        result['fields'] = xfields
+        result['arch'], result['fields'] = cls.parse_view(
+            tree, result['type'], result['field_childs'], level=level)
 
         cls._fields_view_get_cache.set(key, result)
         return result
@@ -354,13 +386,17 @@ class ModelView(Model):
     @classmethod
     def view_toolbar_get(cls):
         """
-        Returns the model specific actions.
+        Returns the model specific actions and exports.
         A dictionary with keys:
             - print: a list of available reports
             - action: a list of available actions
             - relate: a list of available relations
+            - exports: a list of available exports
         """
-        Action = Pool().get('ir.action.keyword')
+        pool = Pool()
+        Action = pool.get('ir.action.keyword')
+        Export = pool.get('ir.export')
+        Email = pool.get('ir.email.template')
         key = cls.__name__
         result = cls._view_toolbar_get_cache.get(key)
         if result:
@@ -368,10 +404,18 @@ class ModelView(Model):
         prints = Action.get_keyword('form_print', (cls.__name__, -1))
         actions = Action.get_keyword('form_action', (cls.__name__, -1))
         relates = Action.get_keyword('form_relate', (cls.__name__, -1))
+        exports = Export.search_read(
+            [('resource', '=', cls.__name__)],
+            fields_names=['name', 'export_fields.name'])
+        emails = Email.search_read(
+            [('model.model', '=', cls.__name__)],
+            fields_names=['name'])
         result = {
             'print': prints,
             'action': actions,
             'relate': relates,
+            'exports': exports,
+            'emails': emails,
             }
         cls._view_toolbar_get_cache.set(key, result)
         return result
@@ -382,15 +426,29 @@ class ModelView(Model):
         return []
 
     @classmethod
-    def _view_look_dom_arch(cls, tree, type, field_children=None):
+    def parse_view(
+            cls, tree, type, field_children=None, level=0, view_depends=None):
+        """
+        Return sanitized XML and the corresponding fields definition
+        """
         pool = Pool()
         ModelAccess = pool.get('ir.model.access')
         FieldAccess = pool.get('ir.model.field.access')
 
         encoder = PYSONEncoder()
-        for xpath, attribute, value in cls.view_attributes():
-            for element in tree.xpath(xpath):
+        if view_depends is None:
+            view_depends = []
+        else:
+            view_depends = view_depends.copy()
+        for xpath, attribute, value, *extra in cls.view_attributes():
+            depends = []
+            if extra:
+                depends, = extra
+            nodes = tree.xpath(xpath)
+            for element in nodes:
                 element.set(attribute, encoder.encode(value))
+            if nodes and depends:
+                view_depends.extend(depends)
 
         fields_width = {}
         tree_root = tree.getroottree().getroot()
@@ -448,7 +506,7 @@ class ModelView(Model):
             for page in tree.xpath('//page[not(descendant::*)]'):
                 page.getparent().remove(page)
 
-        if type == 'tree':
+        if type == 'tree' and Transaction().context.get('view_tree_width'):
             ViewTreeWidth = pool.get('ir.ui.view_tree_width')
             viewtreewidth_ids = ViewTreeWidth.search([
                 ('model', '=', cls.__name__),
@@ -458,8 +516,8 @@ class ModelView(Model):
                 if viewtreewidth.width > 0:
                     fields_width[viewtreewidth.field] = viewtreewidth.width
 
-        fields_def = cls.__view_look_dom(tree_root, type,
-                fields_width=fields_width)
+        fields_def = cls.__parse_fields(
+            tree_root, type, fields_width=fields_width)
 
         if hasattr(cls, 'active'):
             fields_def.setdefault('active', {'name': 'active'})
@@ -471,19 +529,27 @@ class ModelView(Model):
                 if hasattr(field, 'field'):
                     fields_def.setdefault(field.field, {'name': field.field})
 
-        for field_name in list(fields_def.keys()):
+        for depend in view_depends:
+            if depend not in fields_to_remove:
+                fields_def.setdefault(depend, {'name': depend})
+
+        field_names = list(fields_def.keys())
+        while field_names:
+            field_name = field_names.pop()
             if field_name in cls._fields:
                 field = cls._fields[field_name]
             else:
                 continue
             for depend in field.depends:
-                fields_def.setdefault(depend, {'name': depend})
+                if depend not in fields_def:
+                    fields_def[depend] = {'name': depend}
+                    field_names.append(depend)
 
         arch = etree.tostring(
             tree, encoding='utf-8', pretty_print=False).decode('utf-8')
         # Do not call fields_def without fields as it returns all fields
         if fields_def:
-            fields2 = cls.fields_get(list(fields_def.keys()))
+            fields2 = cls.fields_get(list(fields_def.keys()), level=level)
         else:
             fields2 = {}
         for field in fields_def:
@@ -492,21 +558,22 @@ class ModelView(Model):
         return arch, fields2
 
     @classmethod
-    def __view_look_dom(cls, element, type, fields_width=None,
-            fields_attrs=None):
+    def __parse_fields(
+            cls, element, type, fields_width=None, _fields_attrs=None):
         pool = Pool()
         Translation = pool.get('ir.translation')
         ModelData = pool.get('ir.model.data')
         ModelAccess = pool.get('ir.model.access')
         Button = pool.get('ir.model.button')
         User = pool.get('res.user')
+        ActionWindow = pool.get('ir.action.act_window')
 
         if fields_width is None:
             fields_width = {}
-        if not fields_attrs:
+        if _fields_attrs is None:
             fields_attrs = {}
         else:
-            fields_attrs = copy.deepcopy(fields_attrs)
+            fields_attrs = _fields_attrs
 
         def set_view_ids(element):
             view_ids = []
@@ -525,10 +592,10 @@ class ModelView(Model):
             elif hasattr(field, 'get_target'):
                 return field.get_target().__name__
 
-        def get_views(relation, view_ids, mode):
+        def get_views(relation, widget, view_ids, mode):
             Relation = pool.get(relation)
             views = {}
-            if field._type in ['one2many', 'many2many']:
+            if widget in {'one2many', 'many2many'}:
                 # Prefetch only the first view to prevent infinite loop
                 if view_ids:
                     for view_id in view_ids:
@@ -542,7 +609,7 @@ class ModelView(Model):
                         break
             return views
 
-        for attr in ('name', 'icon'):
+        for attr in ('name', 'icon', 'symbol'):
             if not element.get(attr):
                 continue
             fields_attrs.setdefault(element.get(attr), {})
@@ -561,7 +628,8 @@ class ModelView(Model):
                     continue
                 mode = (
                     element.attrib.pop('mode', None) or 'tree,form').split(',')
-                views = get_views(relation, view_ids, mode)
+                widget = element.attrib.get('widget', field._type)
+                views = get_views(relation, widget, view_ids, mode)
                 element.attrib['mode'] = ','.join(mode)
                 fields_attrs[fname].setdefault('views', {}).update(views)
 
@@ -596,7 +664,12 @@ class ModelView(Model):
 
             change = cls.__change_buttons[button_name]
             if change:
-                element.set('change', encoder.encode(list(change)))
+                change = list(change)
+                # Add id to change if the button is not cached
+                # Not having the id increase the efficiency of the cache
+                if cls.__rpc__[button_name].cache:
+                    change.append('id')
+                element.set('change', encoder.encode(change))
             if not is_instance_method(cls, button_name):
                 element.set('type', 'class')
             else:
@@ -605,6 +678,28 @@ class ModelView(Model):
             for depend in states.get('depends', []):
                 fields_attrs.setdefault(depend, {})
 
+        if element.tag == 'link':
+            link_name = element.attrib['name']
+            action_id = ModelData.get_id(*link_name.split('.'))
+            try:
+                with Transaction().set_context(_check_access=True):
+                    action, = ActionWindow.search([('id', '=', action_id)])
+            except ValueError:
+                action = None
+            if (not action
+                    or not action.res_model
+                    or not ModelAccess.check(
+                        action.res_model, 'read', raise_exception=False)):
+                element.tag = 'label'
+                colspan = element.attrib.get('colspan')
+                link_name = element.attrib['name']
+                element.attrib.clear()
+                element.attrib['id'] = link_name
+                if colspan is not None:
+                    element.attrib['colspan'] = colspan
+            else:
+                element.attrib['id'] = str(action.action.id)
+
         # translate view
         if Transaction().language != 'en':
             for attr in ('string', 'sum', 'confirm', 'help'):
@@ -623,14 +718,16 @@ class ModelView(Model):
                     fields_attrs.setdefault(element.get(attr), {})
 
         for field in element:
-            fields_attrs = cls.__view_look_dom(field, type,
-                fields_width=fields_width, fields_attrs=fields_attrs)
+            fields_attrs = cls.__parse_fields(
+                field, type, fields_width=fields_width,
+                _fields_attrs=fields_attrs)
         return fields_attrs
 
     @staticmethod
     def button(func):
         @wraps(func)
         def wrapper(cls, records, *args, **kwargs):
+            from .modelstorage import ModelStorage
             pool = Pool()
             ModelAccess = pool.get('ir.model.access')
             Button = pool.get('ir.model.button')
@@ -644,14 +741,20 @@ class ModelView(Model):
 
             if (transaction.user != 0) and check_access:
                 ModelAccess.check(cls.__name__, 'read')
+                if issubclass(cls, ModelStorage):
+                    # Check record rule access
+                    cls.read([r.id for r in records
+                            if r.id is not None and r.id >= 0],
+                        ['id'])
                 groups = set(User.get_groups())
                 button_groups = Button.get_groups(cls.__name__,
                     func.__name__)
                 if button_groups:
                     if not groups & button_groups:
-                        raise UserError(
-                            'Calling button %s on %s is not allowed!'
-                            % (func.__name__, cls.__name__))
+                        raise AccessButtonError(
+                            gettext('ir.msg_access_button_error',
+                                button=func.__name__,
+                                model=cls.__name__))
                 else:
                     ModelAccess.check(cls.__name__, 'write')
 
@@ -683,20 +786,30 @@ class ModelView(Model):
                 ModelData = pool.get('ir.model.data')
                 Action = pool.get('ir.action')
 
-                func(*args, **kwargs)
+                value = func(*args, **kwargs)
 
                 module, fs_id = action.split('.')
                 action_id = Action.get_action_id(
                     ModelData.get_id(module, fs_id))
-                return action_id
+                if value:
+                    action_value = Action(action_id).get_action_value()
+                    action_value.update(value)
+                    return action_value
+                else:
+                    return action_id
             return wrapper
         return decorator
 
     @staticmethod
-    def button_change(*fields):
+    def button_change(*fields, **kwargs):
+        methods = kwargs.pop('methods', None)
+        assert not kwargs
+
         def decorator(func):
             func = on_change(func)
             func.change = set(fields)
+            if methods:
+                func.change_methods = set(methods)
             return func
         return decorator
 
@@ -732,10 +845,11 @@ class ModelView(Model):
         """
         from .modelstorage import ModelStorage
         changed = {}
-        init_values = self._init_values or {}
+        init_values = self._init_values or self._record()
         if not self._values:
             return changed
-        for fname, value in self._values.items():
+        init_record = self.__class__(self.id)
+        for fname, value in self._values._items():
             field = self._fields[fname]
             # Always test key presence in case value is None
             if (fname in init_values
@@ -745,7 +859,9 @@ class ModelView(Model):
             if field._type in ('many2one', 'one2one', 'reference'):
                 if value:
                     if isinstance(value, ModelStorage):
-                        changed['%s.rec_name' % fname] = value.rec_name
+                        changed['%s.' % fname] = {
+                            'rec_name': value.rec_name,
+                            }
                     if value.id is None:
                         # Don't consider temporary instance as a change
                         continue
@@ -753,37 +869,70 @@ class ModelView(Model):
                         value = str(value)
                     else:
                         value = value.id
-            elif field._type == 'one2many':
+            elif field._type in ['one2many', 'many2many']:
                 targets = value
-                init_targets = list(init_values.get(fname, []))
+                if fname in init_values:
+                    init_targets = init_values._get(fname)
+                else:
+                    init_targets = getattr(init_record, fname, [])
                 value = collections.defaultdict(list)
-                value['remove'] = [t.id for t in init_targets if t.id]
+                previous = [t.id for t in init_targets if t.id]
                 for i, target in enumerate(targets):
-                    if target.id in value['remove']:
-                        value['remove'].remove(target.id)
-                        target_changed = target._changed_values
-                        if target_changed:
-                            target_changed['id'] = target.id
-                            value['update'].append(target_changed)
+                    if (field._type == 'one2many'
+                            and field.field
+                            and target._values):
+                        t_values = target._values._copy()
+                        # Don't look at reverse field
+                        target._values._pop(field.field, None)
                     else:
-                        if isinstance(target, ModelView):
-                            # Ensure initial values are returned because target
-                            # was instantiated on server side.
-                            target_init_values = target._init_values
-                            target._init_values = None
-                            try:
-                                added_values = target._changed_values
-                            finally:
-                                target._init_values = target_init_values
+                        t_values = None
+                    try:
+                        if target.id in previous:
+                            previous.remove(target.id)
+                            if isinstance(target, ModelView):
+                                target_changed = target._changed_values
+                                if target_changed:
+                                    target_changed['id'] = target.id
+                                    value['update'].append(target_changed)
                         else:
-                            added_values = target._default_values
-                        value['add'].append((i, added_values))
-                if not value['remove']:
-                    del value['remove']
+                            if isinstance(target, ModelView):
+                                # Ensure initial values are returned because
+                                # target was instantiated on server side.
+                                target_init_values = target._init_values
+                                target._init_values = None
+                                try:
+                                    added_values = target._changed_values
+                                finally:
+                                    target._init_values = target_init_values
+                            else:
+                                added_values = target._default_values
+                            added_values['id'] = target.id
+                            value['add'].append((i, added_values))
+                    finally:
+                        if t_values:
+                            target._values = t_values
+                if previous:
+                    to_delete, to_remove = [], []
+                    deleted = removed = None
+                    if self._deleted:
+                        deleted = self._deleted[fname]
+                    if self._removed:
+                        removed = self._removed[fname]
+                    for id_ in previous:
+                        if deleted and id_ in deleted:
+                            to_delete.append(id_)
+                        elif removed and id_ in removed:
+                            to_remove.append(id_)
+                        elif field._type == 'one2many':
+                            to_delete.append(id_)
+                        else:
+                            to_remove.append(id_)
+                    if to_delete:
+                        value['delete'] = to_delete
+                    if to_remove:
+                        value['remove'] = to_remove
                 if not value:
                     continue
                 value = dict(value)
-            elif field._type == 'many2many':
-                value = [r.id for r in value]
             changed[fname] = value
         return changed
