@@ -31,8 +31,7 @@ class PYSON(object):
             other = Bool(other)
         if (isinstance(self, And)
                 and not isinstance(self, Or)):
-            self._statements.append(other)
-            return self
+            return And(*self._statements, other)
         if self.types() != {bool}:
             return And(Bool(self), other)
         else:
@@ -45,8 +44,7 @@ class PYSON(object):
                 and other.types() != {bool}):
             other = Bool(other)
         if isinstance(self, Or):
-            self._statements.append(other)
-            return self
+            return Or(*self._statements, other)
         if self.types() != {bool}:
             return Or(Bool(self), other)
         else:
