@@ -389,7 +389,7 @@ class User(avatar_mixin(100, 'login'), DeactivableMixin, ModelSQL, ModelView):
             all_users += users
             args.extend((users, cls._convert_vals(values)))
 
-            if 'password' in values:
+            if values.keys() & {'active', 'password'}:
                 session_to_clear += users
                 users_to_clear += [u.login for u in users]
 
