@@ -3,10 +3,14 @@
 # this repository contains the full copyright notices and license terms.
 import unittest
 
+from sql import Literal
+
 from trytond import backend
-from trytond.exceptions import UserError
+from trytond.model.exceptions import (
+    RequiredValidationError, ForbiddenCharValidationError)
 from trytond.pool import Pool
 from trytond.tests.test_tryton import activate_module, with_transaction
+from trytond.tests.test_tryton import ExtensionTestCase
 from trytond.transaction import Transaction
 
 
@@ -37,6 +41,15 @@ class CommonTestCaseMixin:
 
         self.assertEqual(char.char, "é")
 
+    @with_transaction()
+    def test_create_with_sql_value(self):
+        "Test create with SQL value"
+        Char = self.Char()
+
+        char, = Char.create([{'char': Literal('Foo')}])
+
+        self.assertEqual(char.char, "Foo")
+
     @with_transaction()
     def test_search_equals(self):
         "Test search char equals"
@@ -364,7 +377,7 @@ class FieldCharTestCase(unittest.TestCase, CommonTestCaseMixin):
         "Test create char required without value"
         Char = Pool().get('test.char_required')
 
-        with self.assertRaises(UserError):
+        with self.assertRaises(RequiredValidationError):
             Char.create([{}])
 
     @with_transaction()
@@ -372,7 +385,7 @@ class FieldCharTestCase(unittest.TestCase, CommonTestCaseMixin):
         "Test create char required with empty"
         Char = Pool().get('test.char_required')
 
-        with self.assertRaises(UserError):
+        with self.assertRaises(RequiredValidationError):
             Char.create([{
                         'char': '',
                         }])
@@ -399,6 +412,16 @@ class FieldCharTestCase(unittest.TestCase, CommonTestCaseMixin):
                         'char': "foobar",
                         }])
 
+    @with_transaction()
+    def test_create_invalid_char(self):
+        "Test create char with invalid char"
+        Char = Pool().get('test.char')
+
+        with self.assertRaises(ForbiddenCharValidationError):
+            Char.create([{
+                        'char': "foo\nbar",
+                        }])
+
     @with_transaction()
     def test_write(self):
         "Test write char"
@@ -453,45 +476,35 @@ class FieldCharTranslatedTestCase(unittest.TestCase, CommonTestCaseMixin):
     def Char(self):
         return Pool().get('test.char_translate')
 
+    @with_transaction()
+    def test_translation_default_language_cache(self):
+        """Test set translation for default language does not fill transactional
+        cache with former value"""
+        pool = Pool()
+        Config = pool.get('ir.configuration')
+        Char = self.Char()
 
-@unittest.skipUnless(backend.name() == 'postgresql',
+        with Transaction().set_context(language=Config.get_language()):
+            char, = Char.create([{
+                        'char': "foo",
+                        }])
+
+            char.char = "bar"
+            char.save()
+
+            self.assertEqual(char.char, "bar")
+
+
+@unittest.skipUnless(backend.name == 'postgresql',
     "unaccent works only on postgresql")
-class FieldCharUnaccentedTestCase(unittest.TestCase):
+class FieldCharUnaccentedTestCase(ExtensionTestCase):
     "Test Field Char with unaccented searches"
+    extension = 'unaccent'
 
     @classmethod
     def setUpClass(cls):
-        super(FieldCharUnaccentedTestCase, cls).setUpClass()
         activate_module('tests')
-        cls._activate_extension()
-
-    @classmethod
-    def tearDownClass(cls):
-        super(FieldCharUnaccentedTestCase, cls).tearDownClass()
-        cls._deactivate_extension()
-
-    @classmethod
-    @with_transaction()
-    def _activate_extension(cls):
-        connection = Transaction().connection
-        cursor = connection.cursor()
-        cursor.execute('CREATE EXTENSION "unaccent"')
-        connection.commit()
-        cls._clear_unaccent_cache()
-
-    @classmethod
-    @with_transaction()
-    def _deactivate_extension(cls):
-        connection = Transaction().connection
-        cursor = connection.cursor()
-        cursor.execute('DROP EXTENSION "unaccent"')
-        connection.commit()
-        cls._clear_unaccent_cache()
-
-    @classmethod
-    def _clear_unaccent_cache(cls):
-        Database = backend.get('Database')
-        Database._has_unaccent.clear()
+        super().setUpClass()
 
     @with_transaction()
     def test_normal_search(self):
@@ -562,10 +575,72 @@ class FieldCharUnaccentedTestCase(unittest.TestCase):
             self.assertListEqual(chars_ecole, [char])
 
 
+@unittest.skipUnless(backend.name == 'postgresql',
+    "similarity works only on postgresql")
+class FieldCharSimilarityTestCase(ExtensionTestCase):
+    "Test Field Char with similarity searches"
+    extension = 'pg_trgm'
+
+    @classmethod
+    def setUpClass(cls):
+        activate_module('tests')
+        super().setUpClass()
+
+    def _test_search(self, Model):
+        record1, record2 = Model.create([{
+                    'char': "word",
+                    }, {
+                    'char': "Foo",
+                    }])
+
+        with Transaction().set_context(search_similarity=0.3):
+            self.assertListEqual(Model.search([
+                        ('char', 'ilike', 'two words'),
+                        ]), [record1])
+
+    @with_transaction()
+    def test_search(self):
+        "Test search"
+        pool = Pool()
+        self._test_search(pool.get('test.char'))
+
+    @with_transaction()
+    def test_search_translated(self):
+        "Test search translated"
+        pool = Pool()
+        self._test_search(pool.get('test.char_translate'))
+
+    def _test_order(self, Model):
+        record1, record2 = Model.create([{
+                    'char': "word",
+                    }, {
+                    'char': "Foo",
+                    }])
+        with Transaction().set_context({
+                    '%s.char.order' % Model.__name__: 'foo bar',
+                    }):
+            self.assertListEqual(Model.search([
+                        ], order=[('char', 'DESC')]),
+                [record2, record1])
+
+    @with_transaction()
+    def test_order(self):
+        "Test order"
+        pool = Pool()
+        self._test_order(pool.get('test.char'))
+
+    @with_transaction()
+    def test_order_translated(self):
+        "Test order translated"
+        pool = Pool()
+        self._test_order(pool.get('test.char_translate'))
+
+
 def suite():
     suite_ = unittest.TestSuite()
     loader = unittest.TestLoader()
     suite_.addTests(loader.loadTestsFromTestCase(FieldCharTestCase))
     suite_.addTests(loader.loadTestsFromTestCase(FieldCharTranslatedTestCase))
     suite_.addTests(loader.loadTestsFromTestCase(FieldCharUnaccentedTestCase))
+    suite_.addTests(loader.loadTestsFromTestCase(FieldCharSimilarityTestCase))
     return suite_
