@@ -2,7 +2,8 @@
 # this repository contains the full copyright notices and license terms.
 import unittest
 
-from trytond.exceptions import UserError
+from trytond.model.exceptions import (
+    RequiredValidationError, SizeValidationError)
 from trytond.pool import Pool
 from trytond.tests.test_tryton import activate_module, with_transaction
 
@@ -38,6 +39,21 @@ class CommonTestCaseMixin:
 
         self.assertListEqual(many2manys, [many2many])
 
+    @with_transaction()
+    def test_search_equals_no_link(self):
+        "Test search many2many equals without link"
+        Many2Many = self.Many2Many()
+        many2many, no_link = Many2Many.create([{
+                    'targets': [('create', [{'name': "Target"}])],
+                    }, {
+                    }])
+
+        many2manys = Many2Many.search([
+                ('targets', '=', "Target"),
+                ])
+
+        self.assertListEqual(many2manys, [many2many])
+
     @with_transaction()
     def test_search_non_equals(self):
         "Test search many2many non equals"
@@ -84,6 +100,21 @@ class CommonTestCaseMixin:
 
         self.assertListEqual(many2manys, [many2many1])
 
+    @with_transaction()
+    def test_search_non_equals_no_link(self):
+        "Test search many2many non equals without link"
+        Many2Many = self.Many2Many()
+        many2many, no_link = Many2Many.create([{
+                    'targets': [('create', [{'name': "Target"}])],
+                    }, {
+                    }])
+
+        many2manys = Many2Many.search([
+                ('targets', '!=', "Target"),
+                ])
+
+        self.assertListEqual(many2manys, [no_link])
+
     @with_transaction()
     def test_search_in(self):
         "Test search many2many in"
@@ -212,6 +243,21 @@ class CommonTestCaseMixin:
 
         self.assertListEqual(many2manys, [])
 
+    @with_transaction()
+    def test_search_not_where_others(self):
+        "Test search many2many not where with others"
+        Many2Many = self.Many2Many()
+        many2many, = Many2Many.create([{
+                    'targets': [('create', [
+                                {'name': "Target"}, {'name': "Foo"}])],
+                    }])
+
+        many2manys = Many2Many.search([
+                ('targets', 'not where', [('name', '=', "Target")]),
+                ])
+
+        self.assertListEqual(many2manys, [])
+
     @with_transaction()
     def test_write_write(self):
         "Test write many2many write"
@@ -308,6 +354,25 @@ class CommonTestCaseMixin:
         self.assertTupleEqual(many2many.targets, (target2,))
         self.assertListEqual(targets, [target2])
 
+    @with_transaction()
+    def test_write_not_readd(self):
+        "Test write many2many do not re-add existing"
+        pool = Pool()
+        Many2Many = self.Many2Many()
+        Relation = pool.get(Many2Many.targets.relation_name)
+        many2many, = Many2Many.create([{
+                    'targets': [('create', [{}])],
+                    }])
+
+        target, = many2many.targets
+
+        Many2Many.write([many2many], {
+                    'targets': [('add', {target.id})],
+                    })
+
+        relation, = Relation.search([])
+        self.assertIsNone(relation.write_date)
+
 
 class FieldMany2ManyTestCase(unittest.TestCase, CommonTestCaseMixin):
     "Test Field Many2Many"
@@ -342,7 +407,7 @@ class FieldMany2ManyTestCase(unittest.TestCase, CommonTestCaseMixin):
         "Test create many2many required without value"
         Many2Many = Pool().get('test.many2many_required')
 
-        with self.assertRaises(UserError):
+        with self.assertRaises(RequiredValidationError):
             Many2Many.create([{}])
 
     @with_transaction()
@@ -363,7 +428,7 @@ class FieldMany2ManyTestCase(unittest.TestCase, CommonTestCaseMixin):
         "Test create many2many size invalid"
         Many2Many = Pool().get('test.many2many_size')
 
-        with self.assertRaises(UserError):
+        with self.assertRaises(SizeValidationError):
             Many2Many.create([{
                         'targets': [
                             ('create', [{}] * 4),
@@ -624,6 +689,43 @@ class FieldMany2ManyTestCase(unittest.TestCase, CommonTestCaseMixin):
                     self.first1, self.first2, self.first3, self.first4,
                     self.second1, self.second2, self.second3, self.second4]))
 
+    @with_transaction()
+    def test_context_attribute(self):
+        "Test context on many2many attribute"
+        pool = Pool()
+        Many2Many = pool.get('test.many2many_context')
+
+        record, = Many2Many.create([{
+                    'targets': [('create', [{}])],
+                    }])
+
+        self.assertEqual(record.targets[0].context, 'foo')
+
+    @with_transaction()
+    def test_context_read(self):
+        "Test context on many2many read"
+        pool = Pool()
+        Many2Many = pool.get('test.many2many_context')
+
+        record, = Many2Many.create([{
+                    'targets': [('create', [{}])],
+                    }])
+        data, = Many2Many.read([record.id], ['targets.context'])
+
+        self.assertEqual(data['targets.'][0]['context'], 'foo')
+
+    @with_transaction()
+    def test_context_set(self):
+        "Test context on many2many set"
+        pool = Pool()
+        Many2Many = pool.get('test.many2many_context')
+        Target = pool.get('test.many2many_context.target')
+
+        target, = Target.create([{}])
+        record = Many2Many(targets=[target.id])
+
+        self.assertEqual(record.targets[0].context, 'foo')
+
 
 class FieldMany2ManyReferenceTestCase(unittest.TestCase, CommonTestCaseMixin):
     "Test Field Many2Many Reference"
