@@ -488,6 +488,10 @@ class PYSONTestCase(unittest.TestCase):
         eval = pyson.PYSONEncoder().encode(pyson.In('test', []))
         self.assertFalse(pyson.PYSONDecoder().decode(eval))
 
+        eval = pyson.PYSONEncoder().encode(
+            pyson.In('test', pyson.Eval('foo', [])))
+        self.assertFalse(pyson.PYSONDecoder({'foo': None}).decode(eval))
+
         self.assertEqual(repr(pyson.In('foo', ['foo', 'bar'])),
             "In('foo', ['foo', 'bar'])")
 
@@ -838,6 +842,19 @@ class PYSONTestCase(unittest.TestCase):
                 ]:
             self.assertEqual(pyson.PYSONDecoder(ctx).decode(eval), result)
 
+    def test_Eval_dot_notation_in_context(self):
+        "Test pyson.Eval with dot notation in context"
+
+        eval = pyson.PYSONEncoder().encode(pyson.Eval('foo.bar', 0))
+        ctx = {
+            'foo.bar': 1,
+            'foo': {
+                'bar': 0,
+                },
+            }
+
+        self.assertEqual(pyson.PYSONDecoder(ctx).decode(eval), 1)
+
     def test_eval_true(self):
         "Test PYSON.eval JS true"
         self.assertEqual(eval('true', pyson.CONTEXT), True)
