@@ -1,13 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
+from trytond.pool import Pool
+from trytond.transaction import Transaction
 from trytond.wizard import Wizard, StateView, StateTransition, StateAction, \
     Button
-from trytond.transaction import Transaction
-
-__all__ = [
-    'TestWizardStart', 'TestWizard',
-    ]
 
 
 class TestWizardStart(ModelSQL, ModelView):
@@ -50,3 +47,12 @@ class TestWizard(Wizard):
     @staticmethod
     def transition_action():
         return 'end'
+
+
+def register(module):
+    Pool.register(
+        TestWizardStart,
+        module=module, type_='model')
+    Pool.register(
+        TestWizard,
+        module=module, type_='wizard')
