@@ -223,6 +223,8 @@ class Transaction(object):
                 for datamanager in self._datamanagers:
                     datamanager.tpc_vote(self)
             self.started_at = self.monotonic_time()
+            for cache in self.cache.values():
+                cache.clear()
             Cache.commit(self)
             self.connection.commit()
         except Exception:
